@@ -2,7 +2,6 @@
 Functions to convert data types into ctypes friendly formats.
 """
 import numpy as np
-import pandas
 
 from ..exceptions import GMTInvalidInput
 
@@ -46,8 +45,8 @@ def dataarray_to_matrix(grid):
     --------
 
     >>> from pygmt.datasets import load_earth_relief
-    >>> # Use the global Earth relief grid with 1 degree spacing (60')
-    >>> grid = load_earth_relief(resolution='60m')
+    >>> # Use the global Earth relief grid with 1 degree spacing
+    >>> grid = load_earth_relief(resolution='01d')
     >>> matrix, region, inc = dataarray_to_matrix(grid)
     >>> print(region)
     [-180.0, 180.0, -90.0, 90.0]
