@@ -105,7 +105,15 @@ class Figure(BasePlotting):
         return wesn
 
     @fmt_docstring
-    @use_alias(F="prefix", T="fmt", A="crop", E="dpi")
+    @use_alias(
+        A="crop",
+        C="gs_option",
+        E="dpi",
+        F="prefix",
+        I="icc_gray",
+        T="fmt",
+        Q="anti_aliasing",
+    )
     @kwargs_to_strings()
     def psconvert(self, **kwargs):
         """
@@ -116,7 +124,7 @@ class Figure(BasePlotting):
 
         If no input files are given, will convert the current active figure
         (see :func:`pygmt.figure`). In this case, an output name must be given
-        using parameter *F*.
+        using parameter *prefix*.
 
         Full option list at :gmt-docs:`psconvert.html`
 
@@ -124,37 +132,38 @@ class Figure(BasePlotting):
 
         Parameters
         ----------
-        A : str or bool
+        crop : str or bool
             Adjust the BoundingBox and HiResBoundingBox to the minimum required
             by the image content. Append ``u`` to first remove any GMT-produced
             time-stamps. Default is True.
-        C : str
+        gs_option : str
             Specify a single, custom option that will be passed on to
             GhostScript as is.
-        E : int
+        dpi : int
             Set raster resolution in dpi. Default = 720 for PDF, 300 for
             others.
-        F : str
+        prefix : str
             Force the output file name. By default output names are constructed
             using the input names as base, which are appended with an
             appropriate extension. Use this option to provide a different name,
             but without extension. Extension is still determined automatically.
-        I : bool
+        icc_gray : bool
             Enforce gray-shades by using ICC profiles.
-        Q : str
+        anti_aliasing : str
             Set the anti-aliasing options for graphics or text. Append the size
             of the subsample box (1, 2, or 4) [4]. Default is no anti-aliasing
             (same as bits = 1).
-        T : str
-            Sets the output format, where b means BMP, e means EPS, E means EPS
-            with PageSize command, f means PDF, F means multi-page PDF, j means
-            JPEG, g means PNG, G means transparent PNG (untouched regions are
-            transparent), m means PPM, s means SVG, and t means TIFF [default
-            is JPEG]. To bjgt you can append - in order to get a grayscale
+        fmt : str
+            Sets the output format, where *b* means BMP, *e* means EPS, *E*
+            means EPS with PageSize command, *f* means PDF, *F* means
+            multi-page PDF, *j* means JPEG, *g* means PNG, *G* means
+            transparent PNG (untouched regions are transparent), *m* means PPM,
+            *s* means SVG, and *t* means TIFF [default is JPEG]. To ``'bjgt'``
+            you can append ``'+m'`` in order to get a monochrome (grayscale)
             image. The EPS format can be combined with any of the other
             formats. For example, ``'ef'`` creates both an EPS and a PDF file.
-            The ``'F'`` creates a multi-page PDF file from the list of input PS
-            or PDF files. It requires the *F* option.
+            Using ``'F'`` creates a multi-page PDF file from the list of input
+            PS or PDF files. It requires the *prefix* option.
 
         """
         kwargs = self._preprocess(**kwargs)
